/**
 * @beta
 * Type representing a successful API result.
 * @template T - The type of the data returned.
 */
export type ApiSuccessResult<T> = {
    /** The status of the API result, always 'success' for this type. */
    status: 'success';
    /** The data returned by the API. */
    data: T;
};
/**
 * @beta
 * Type representing an error API result.
 */
export type ApiErrorResult = {
    /** The status of the API result, always 'error' for this type. */
    status: 'error';
    /** The error message or data. */
    data: string;
    /** The cause of the error, represented by a number. */
    cause: number;
};
/**
 * @beta
 * Type representing a general API result which can be either success or error.
 * @template T - The type of the data returned in case of success.
 */
export type ApiResult<T> = ApiSuccessResult<T> | ApiErrorResult;
//# sourceMappingURL=api.d.ts.map