/**
 * @beta
 * Type representing a callback function.
 * @template T - The type of the parameter for the callback function.
 * @param t - The parameter for the callback function.
 */
export type Callback<T> = (t: T) => void;
/**
 * @beta
 * Type representing an unsubscribe function.
 * @returns A promise that resolves when the unsubscribe operation is complete.
 */
export type Unsubscribe = () => Promise<void>;
/**
 * @beta
 * Type representing a function to add a listener.
 * @template T - The type of the parameter for the callback function.
 * @param callback - The callback function to be invoked.
 * @returns A promise that resolves to an unsubscribe function.
 */
export type AddListener<T> = (callback: Callback<T>) => Promise<Unsubscribe>;
//# sourceMappingURL=callback.d.ts.map