/**
 * @beta
 * Type representing a filter definition that is provided.
 */
export type ProvidedFilterDefinition = {
    /** The type of the filter, always 'provided' for this type. */
    type: 'provided';
};
/**
 * @beta
 * Type representing a JavaScript filter definition.
 */
export type JavascriptFilterDefinition = {
    /** The type of the filter, always 'javascript' for this type. */
    type: 'javascript';
    /** The JavaScript code defining the filter. */
    code: string;
};
/**
 * @beta
 * Type representing a semantically bound filter definition.
 */
export type SemanticallyBoundFilterDefinition = {
    /** The type of the filter, always 'semantically_bound' for this type. */
    type: 'semantically_bound';
    /** The concepts that the filter is bound to. */
    concepts: string[];
};
/**
 * @beta
 * Type representing the name of a filter.
 */
export type FilterName = string;
/**
 * @beta
 * Type representing a filter definition, which can be one of several specific filter types.
 */
export type FilterDefinition = ProvidedFilterDefinition | JavascriptFilterDefinition | SemanticallyBoundFilterDefinition;
//# sourceMappingURL=filters.d.ts.map